#!-PERL-
# File:		mifren.pl
# Version:	1.0
# Copyright:	MacDonald Dettwiler and Associates, 1994
# Author:	Duncan Fraser (djf@mda.ca)
# This file may be modified provided this header is kept intact
# and all modifications are documented here.
# This code may not be included in a commercial product without
# the permission of the author.

# This script is part of the fm2html package.

$\ = "\n";		# set output record separator
$, = " ";		# set output field separator
$* = 0;			# set single line pattern matching

select( STDERR );	# all error messages go to stderr
$| = 1;			# make stderr unbuffered

$FALSE = 0;
$TRUE  = 1;

$MIF_MAGIC		= "<MIFFile";
$MIF_BOOK_MAGIC		= "<Book ";
$FM_MAGIC		= "<MakerFile";
$FM_BOOK_MAGIC		= "<Bookfile";

$PTAGS_EXTENSION	= "ptags";
$CTAGS_EXTENSION	= "ctags";

$TMPNAME		= "mifren.tmp";


sub escape	# take string, insert backslash before special characters
{
  local( $_ ) = @_;
  s|([\.\+\?\*\'\"\(\)])|\\$1|g;
  $_;
}

sub dirname	# take pathname, return leading directories
{
  local( $_ ) = @_;
  s|^(.+)/.*$|$1| || s|^/[^/]*$|/| || s|^[^/]*$|.|;
  $_;
}

sub basename	# take pathname, strip leading directories and optional suffix
{
  local( $_, $suffix ) = @_;
  m|/|o && s|^.*/([^/]*)$|$1|;
  $suffix = &escape( $suffix );
  s|(.*)$suffix|$1|;
  $_;
}


local( $prog ) = &basename( $0 );
local( $usage ) = "Usage: $prog [-p|-s] [-o old] [-n new] [-q] [-r] filename";

local( $newfix, $oldfix, $position, $tmpdir, $tmpfile,
       $infile, $indir, $cwd, @errs, @oldmifpaths, $oldmifpath,
       %olddocnames, %newdocnames, %newdocpaths );

local( $echo ) = $FALSE;
local( $quiet ) = $FALSE;
local( $remove ) = $FALSE;
local( $is_infile ) = $TRUE;
local( $exitval ) = 0;

while ( $ARGV[0] =~ m|^-| )
{
  $_ = shift;
  last if m|^--|;
  
  if ( m|^-e$| )
  {
    $echo = $TRUE; next;
  }
  elsif ( m|^-n$| )
  {
    $newfix = "$ARGV[0]"; shift; next;
  }
  elsif ( m|^-o$| )
  {
    $oldfix = "$ARGV[0]"; shift; next;
  }
  elsif ( m|^-p$| )
  {
    if ( "$position" eq "SUFFIX" )
    {
      print
	"$prog: ERROR: Position may be either prefix or suffix, but not both";
      print "$usage";
      exit( 1 );
    }
    $position = "PREFIX"; next;
  }
  elsif ( m|^-q$| )
  {
    $quiet = $TRUE; next;
  }
  elsif ( m|^-r$| )
  {
    $remove = $TRUE; next;
  }
  elsif ( m|^-s$| )
  {
    if ( "$position" eq "PREFIX" )
    {
      print
	"$prog: ERROR: Position may be either prefix or suffix, but not both";
      print "$usage";
      exit( 1 );
    }
    $position = "SUFFIX"; next;
  }

  print "$prog: ERROR: Invalid flag: \"$_\"";
  print "$usage";
  exit( 1 );
}

if ( $#ARGV != 0 )
{
  print "$prog: ERROR: You must specify exactly one MIF book or document to rename";
  print "$usage";
  exit( 1 );
}

if ( ! "$oldfix" && ! "$newfix" )
{
  print "$prog: ERROR: You must specify old text (to be stripped) or new text (to be prepended/appended) or both";
  print "$usage";
  exit( 1 );
}

if ( ! "$position" )
{
  print "$prog: ERROR: You must specify the position with either \"-p\" (prefix) or \"-s\" (suffix)";
  print "$usage";
  exit( 1 );
}

$quiet == $FALSE && print "";

$infile = "$ARGV[0]";

if ( ! -e "$infile" )
  { die "$prog: Input file \"$infile\" does not exist... exiting"; }
if ( ! open( INFILE, "<$infile" ) )
  { die "$prog: Can't open input file \"$infile\"... exiting"; }

$magic = <INFILE>;

if ( $magic =~ m|^$FM_MAGIC|i || $magic =~ m|^$FM_BOOK_MAGIC|i )
  { die "$prog: FrameMaker file \"$infile\" is not a MIF file... exiting"; }
if ( $magic !~ m|^$MIF_MAGIC|i && $magic !~ m|^$MIF_BOOK_MAGIC|i )
  { die "$prog: Input file \"$infile\" is not a FrameMaker file... exiting";}

close( INFILE );

# Docs and xrefs may have relative pathnames so it is necessary to chdir.
# Unfortunately, however, FrameMaker sometimes has problems figuring out
# what the current directory is and defaults to the user's home directory,
# in which case it won't find the files anyway.
$indir = &dirname( "$infile" );
chdir( "$indir" ) ||
  die "mif2mif: ERROR: Can't change directory to \"$indir\"... exiting";
chop( $cwd = `pwd` );
$infile = &basename( "$infile" );
$infile = "$cwd/$infile";

$tmpdir = $ENV{'TMPDIR'};
if ( ! $tmpdir ) { $tmpdir = "-TMPDIR-"; }
if ( ! $tmpdir ) { $tmpdir = "$cwd"; }
$tmpdir =~ s|/$||;
$tmpfile = "$tmpdir/$TMPNAME.$$";

if ( $magic =~ m|^$MIF_BOOK_MAGIC|i )
{
  open( INFILE, "<$infile" ) ||
    die "$prog: Can't open input file \"$infile\"... exiting";
  $oldmifpaths[++$#oldmifpaths] = "$infile";

  while ( <INFILE> )
  {
    local( $mifpath );

    if ( m|<FileName | )
    {
      if ( m|<FileName \s*`<c\\>[^<]*<U\\>([^']*)'>| )
      {
        $docpath = "$1";	# FrameMaker 3.1
      }
      elsif ( m|<FileName \s*`<c\\>([^<]*)'>| )
      {
        $docpath = "$1";	# FrameMaker 4.0
      }
      
      $mifpath = "$docpath.mif";

      if ( ! -e "$mifpath" )
      {
	local( $unstripped ) = "$mifpath";

	$mifpath =~ s|\.doc||;

	if ( $mifpath eq $unstripped )
	{
          $errs[++$#errs] = 
	    "$prog: ERROR: Input file \"$unstripped\" does not exist";
          next;
	}
	elsif ( ! -e $mifpath )
	{
          $errs[++$#errs] = "$prog: ERROR: Input file \"$unstripped\" does not exist, neither does \"$mifpath\"";
          next;
	}
      }

      $oldmifpaths[++$#oldmifpaths] = "$mifpath";
    }
  }

  close( INFILE );

  @ARGV = @oldmifpaths;
}
else
{
  $ARGV[0] = &basename( "$infile" );
  @oldmifpaths = @ARGV;
}

if ( @errs )
{
  local( $err );
  while ( $err = shift( @errs ) )
    { print "$err"; }
  die "\n$prog: Can't proceed due to error(s)... exiting";
}

while ( $oldmifpath = shift )
{
  local( $olddocname, $newdocname, $newmifpath, $dir, $base, $extension );

  $dir = &dirname( "$oldmifpath" );
  $base = &basename( "$oldmifpath" );

  $olddocname = "$base";
  $olddocname =~ s|\.mif$||;
  $olddocnames{$oldmifpath} = "$olddocname";

  $newdocname = "$olddocname";
  $newdocname =~ s|\.doc$|| ? ($extension = ".doc") : ($extension = "");

  if ( "$oldfix" )
  {
    if ( "$oldfix" eq "$newdocname" && ! "$newfix" )
    {
      $errs[++$#errs] = 
	"$prog: ERROR: Can't strip entire filename of \"$oldmifpath\"";
      $is_infile = $FALSE;
      next;
    }
    else
    {
      local( $escaped ) = &escape( "$oldfix" );

      if ( "$position" eq "PREFIX" )
	{ $newdocname =~ s|^$escaped||; }
      else
	{ $newdocname =~ s|$escaped$||; }
    }
  }

  if ( "$newfix" )
  {
    if ( "$position" eq "PREFIX" )
      { $newdocname = "$newfix$newdocname"; }
    else
      { $newdocname = "$newdocname$newfix"; }
  }

  $newdocname = "$newdocname$extension";
  $newdocnames{$oldmifpath} = "$newdocname";
  $newdocpaths{$oldmifpath} = "$dir/$newdocname";

  $newmifpath = "$dir/$newdocname.mif";
  $newmifpaths{$oldmifpath} = "$newmifpath";

  if ( -e "$newmifpath" )
  {
    $errs[++$#errs] = 
      "$prog: ERROR: Can't rename \"$oldmifpath\" as \"$newmifpath\" because file already exists";
  }

  if ( $is_infile == $TRUE && $echo == $TRUE )
  {
    print STDOUT "$newmifpath";
    $is_infile = FALSE;
  }
}

if ( @errs )
{
  local( $err );
  while ( $err = shift( @errs ) )
    { print "$err"; }
  die "\n$prog: Can't proceed due to error(s)... exiting";
}


open( TMPF, ">$tmpfile" ) || die "Can't open temporary file \"$tmpfile\"";

print TMPF <<END;
#!-PERL-

\$FALSE = 0;
\$TRUE  = 1;

sub substitute
{
  local( \$inf, \$outf ) = \@_;
  local( \$magic, \$\\ );

  \$\\ = \"\\n\";

  if ( ! -e \$inf )
  {
    print \"$prog: ERROR: Can't convert \\\"\$inf\\\" (does not exist)\";
    return 1;
  }

  if ( ! open( INF, \"<\$inf\" ) )
  {
    print \"$prog: ERROR: Can't open file \\\"\$inf\\\"\";
    return 1;
  }

  if ( ! open( OUTF, \">\$outf\" ) )
  {
    print \"$prog: ERROR: Can't create file \\\"\$outf\\\"\";
    return 1;
  }

  \$magic = <INF>;

  if ( \$magic !~ m|$MIF_MAGIC|i && \$magic !~ m|$MIF_BOOK_MAGIC|i )
  {
    print \"$prog: ERROR: File \\\"\$inf\\\" is not a MIF file\";
    return 1;
  }

  chop( \$magic ); print OUTF \$magic;

  while ( <INF> )
  {
    if ( m|<XRefSrcFile |o )
    {
END

@ARGV = @oldmifpaths;
while ( $oldmifpath = shift )
{
  local( $from ) = $olddocnames{$oldmifpath};
  local( $escaped ) = &escape( $from );
  local( $to ) = $newdocnames{$oldmifpath};

  print TMPF "      s|`<c\\\\>$escaped'|`<c\\\\>$to'|o;";
}

print TMPF <<END;
    }
    elsif ( m|<FileName |o )
    {
END

@ARGV = @oldmifpaths;
while ( $oldmifpath = shift )
{
  local( $from ) = $olddocnames{$oldmifpath};
  local( $escaped ) = &escape( $from );
  local( $to ) = $newdocnames{$oldmifpath};
  local( $to_path ) = $newdocpaths{$oldmifpath};

  # FrameMaker 3.1
  print TMPF "      s|`<c\\\\>$escaped<U\\\\>[^']*'|`<c\\\\>$to<U\\\\>$to_path'|o;";
  # FrameMaker 4.0
  print TMPF "      s|`<c\\\\>$escaped'|`<c\\\\>$to'|o;";
}

print TMPF <<END;
    }

    chop; print OUTF;
  }

  0;	# successful return
}
END

close( TMPF );
require( "$tmpfile" );


@ARGV = @oldmifpaths;

while ( $oldmifpath = shift )
{
  local( $retval );

  $quiet == $FALSE && print
    "$prog: Converting \"$oldmifpath\" to \"$newmifpaths{$oldmifpath}\"...";
  $retval = &substitute( $oldmifpath, $newmifpaths{$oldmifpath} );
  $retval != 0 && ($exitval = $retval);

  if ( $remove == $TRUE )
  {
    $quiet == $FALSE &&
      print "mif2mif: Deleting original file \"$oldmifpath\"...";
    eval system( "rm -f $oldmifpath" );
  }
}

eval system( "rm -f $tmpfile" );

$quiet == $FALSE && print "$prog: Done\n";

exit( $exitval );
